use crate::{
    errors::HandlerUsed,
    handlers::{markups::deletion_markup, utils::delete_optional},
    PinnedFuture,
};
use sea_orm::prelude::*;
use teloxide::{adaptors::Throttle, prelude::*};

pub async fn generic<F>(
    bot: Throttle<Bot>,
    text: String,
    msg: Message,
    db: DatabaseConnection,
    dialogue: crate::handlers::MainDialogue,
    check: F,
    next: crate::handlers::PackagedHandler<String>,
) -> crate::Result<()>
where
    for<'a> F: FnOnce(
        &'a Throttle<Bot>,
        &'a Message,
        &'a DatabaseConnection,
        &'a str,
    ) -> PinnedFuture<'a, crate::Result<bool>>,
{
    let handler = next.lock().await.take();
    if text == "/cancel" {
        let previous = handler.and_then(|handler| handler.previous);
        delete_optional(&bot, &previous).await;
        bot.send_message(msg.chat.id, "Successfully cancelled")
            .reply_markup(deletion_markup())
            .await?;
        return Ok(());
    }
    let handler = handler.ok_or(HandlerUsed)?;
    delete_optional(&bot, &handler.previous).await;
    match check(&bot, &msg, &db, &text).await {
        Ok(true) => (),
        Ok(false) => {
            dialogue.exit().await?;
            return Ok(());
        }
        Err(err) => {
            let _ = dialogue.exit().await;
            return Err(err);
        }
    };
    (handler.handler)(bot, msg, db, dialogue, text).await
}
