use crate::{
    entity::prelude::MasterPass,
    errors::{NoMessageText, NoUserInfo},
    handlers::{markups::deletion_markup, MainDialogue, PackagedHandler},
};
use sea_orm::prelude::*;
use teloxide::{adaptors::Throttle, prelude::*};

#[inline]
pub async fn check_master_pass<'a>(
    bot: &'a Throttle<Bot>,
    msg: &'a Message,
    db: &'a DatabaseConnection,
    master_pass: &'a str,
) -> crate::Result<bool> {
    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;
    let result = MasterPass::verify_master_pass(user_id, master_pass.to_owned(), db).await;
    match result {
        Ok(Some(true)) => Ok(true),
        Ok(Some(false)) => {
            bot.send_message(msg.chat.id, "Wrong master password")
                .reply_markup(deletion_markup())
                .await?;
            Ok(false)
        }
        Ok(None) => {
            bot.send_message(msg.chat.id, "No master password set")
                .reply_markup(deletion_markup())
                .await?;
            Ok(false)
        }
        Err(err) => Err(err),
    }
}

pub async fn get_master_pass(
    bot: Throttle<Bot>,
    msg: Message,
    db: DatabaseConnection,
    dialogue: MainDialogue,
    next: PackagedHandler<String>,
) -> crate::Result<()> {
    let text = msg.text().ok_or(NoMessageText)?.trim().to_owned();
    super::generic::generic(
        bot,
        text,
        msg,
        db,
        dialogue,
        |bot, msg, db, text| Box::pin(check_master_pass(bot, msg, db, text)),
        next,
    )
    .await
}
